﻿namespace Dsl.Demo

import System
import System.IO
import System.Reflection

import Boo.Lang.Compiler
import Boo.Lang.Compiler.Ast
import Boo.Lang.Compiler.IO
import Boo.Lang.Compiler.Pipelines
import Boo.Lang.Compiler.Steps


class CompilerBootStrapping:
	
	[getter(GeneratedType)]
	_type as Type

	[getter(Result)]
	_result as CompilerContext

	[getter(References)]
	_references = List[of Assembly]()
	
	[getter(Namespaces)]
	_namespaces = System.Collections.Specialized.StringCollection()
	
	[property(BaseTypeName)]
	_baseTypeName as string
	
	Errors as CompilerErrorCollection:
		get:
			return _result.Errors
	
	def Load([required] file as string) as bool:
		result = CompileScript(file)
		return ProcessResult(result, file)
		
	private def CompileScript(file as string) as CompilerContext:
		compiler = BooCompiler()
		compiler.Parameters.Ducky = true
		compiler.Parameters.Pipeline = CompileToFile()
		compiler.Parameters.Input.Add(FileInput(file))
		compiler.Parameters.OutputType = CompilerOutputType.Library
		
		for reference as Assembly in _references:
			compiler.Parameters.References.Add(reference)

		pipeline = compiler.Parameters.Pipeline
		pipeline.Insert(1, GlobalsToBaseClassExecute(_baseTypeName, _namespaces ))
		return compiler.Run()		

	protected def ProcessResult(result as CompilerContext, file as string):
		typeName = Path.GetFileNameWithoutExtension(file)
		_result = result
		
		if len(result.Errors) == 0:
			_type = _result.GeneratedAssembly.GetType(typeName)
			
		return len(result.Errors) == 0

